////////////////////////////////////////////////////////////////
//Kapitel Bibliotheken
//Beispiel für toxiclips Geometrie- und Physikbibliothek
//Sechsfachpendel mit äußerer Kraft
///////////////////////////////////////////////////////////////
import toxi.physics2d.*;//für VerletPhysics2D
import toxi.physics2d.behaviors.*;//für AttractionBehavior2D 
import toxi.geom.*; //für Vec2D 

VerletPhysics2D physics;
AttractionBehavior2D mouseAttractor;
AttractionBehavior2D zufallAttractor;
int i =0;
float laenge  = 75;//Federlänge
int anzahlMassen =6;
float fHaerte = 1.9;//Federhärte
Vec2D mausPos, zufallPos;
boolean kraftZeichnen = false;
VerletSpring2D [] feder = new VerletSpring2D[anzahlMassen];
Masse [] m = new Masse[anzahlMassen+1] ;
color farbe;//Farbe der Massen

void setup(){
  size(1000, 1000);
  background(255);
  stroke(70);
  //Massenpunkte initialisieren
  m[0] = new Masse(new Vec2D(width/2, height/2),0,10,color(0));
  m[0].lock();//Aufhängung: m[0] bewegt sich nicht
  for(int i =1;i<=anzahlMassen;i++){
    farbe = color(random(255),random(255),random(255));
    m[i]= new Masse(new Vec2D(width/2+random(3),height/2-i*laenge),1,40,farbe);
  }
  //Federstücke initialisieren
  for(int i =0;i<anzahlMassen;i++){
    feder[i]= new VerletSpring2D(m[i], m[i+1], laenge, fHaerte);
  }
  //Physik-Bibliothek initialisieren
  physics = new VerletPhysics2D();
  physics.setWorldBounds(new Rect(0, 0, width, height));
  
  //Gravitationskraft F = (0,0.1) zeigt senkrecht nach unten
  physics.addBehavior(new GravityBehavior2D(new Vec2D(0, 0.1)));//(0,0.1)
  
  //Massenpunkte und Federn in die Welt bringen
  for(int i =0;i<anzahlMassen+1;i++){
    physics.addParticle(m[i]);
  }
  for(int i =0;i<anzahlMassen;i++){
     physics.addSpring(feder[i]);
  }
}

void draw(){
  background(255);
  //Positionen aller Massenpunkte und Federn im Laufe der Zeit
  i++; 
  i=i%400; 
  if(i==0){ //Jede 400. draw-Durchgang Kraft auf Zufallsposition
    zufallPos = new Vec2D(random(100,900), random(100,300));
    // Kraftfeld erzeugen: Ort, Radius, Stärke
    zufallAttractor = new AttractionBehavior2D(zufallPos, 1000, 1);
    physics.addBehavior(zufallAttractor);
    kraftZeichnen = true;
  }
  if(i==100){//Nach 100 draw-Durchgängen Zufallskraftfeld entfernen  
    physics.removeBehavior(zufallAttractor);
    kraftZeichnen = false;
  }

  if(kraftZeichnen){
    fill(0);
    ellipse(zufallPos.x,zufallPos.y,20,20);
  }
   if(mousePressed){
    fill(0);
    ellipse(mausPos.x,mausPos.y,20,20);  
  }
  physics.update();
  strokeWeight(6); 
  for(int i =0;i<anzahlMassen;i++){
     line(m[i].x,m[i].y,m[i+1].x,m[i+1].y);
  }
  for(int i=0;i<=anzahlMassen; i++){
    m[i].display();
  }
}

void mousePressed(){
   mausPos = new Vec2D(mouseX, mouseY);
  // Kraftfeld an mausPos mit Stärke 0.9 und Wirkweite 900
   mouseAttractor = new AttractionBehavior2D(mausPos, 900, 3);
   physics.addBehavior(mouseAttractor);
}

void mouseDragged() {
  // Kraftfeld Position updaten
  mausPos.set(mouseX, mouseY);
}

void mouseReleased() {
  // Kraftfeld entfernen
  physics.removeBehavior(mouseAttractor);
}


void keyPressed(){
  if(key=='s'){
    saveFrame("Pendel.jpg");
  }
}